/*********************************************************************************************************
* Software License Agreement (BSD License)                                                               *
* Author: Sebastien Decugis <sdecugis@nict.go.jp>							 *
*													 *
* Copyright (c) 2008, WIDE Project and NICT								 *
* All rights reserved.											 *
* 													 *
* Redistribution and use of this software in source and binary forms, with or without modification, are  *
* permitted provided that the following conditions are met:						 *
* 													 *
* * Redistributions of source code must retain the above 						 *
*   copyright notice, this list of conditions and the 							 *
*   following disclaimer.										 *
*    													 *
* * Redistributions in binary form must reproduce the above 						 *
*   copyright notice, this list of conditions and the 							 *
*   following disclaimer in the documentation and/or other						 *
*   materials provided with the distribution.								 *
* 													 *
* * Neither the name of the WIDE Project or NICT nor the 						 *
*   names of its contributors may be used to endorse or 						 *
*   promote products derived from this software without 						 *
*   specific prior written permission of WIDE Project and 						 *
*   NICT.												 *
* 													 *
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED *
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A *
* PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR *
* ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 	 *
* LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 	 *
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR *
* TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF   *
* ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.								 *
*********************************************************************************************************/

/* This file contains the definition of our test harness.
 * The harness is very simple yet.
 * It may be interessant to go to dejagnu later...
 *
 */
#ifndef _TESTS_H
#define _TESTS_H

/* Standard includes */
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>
#include <string.h>

/* The waaad API */
#include <waaad/waaad.h>

/* The waaad internal files */
#include <waaad-internal.h>


/* Define the return code values */
#define PASS	0
#define FAIL	1

/* Define the macro to fail a test with a message */
#define FAILTEST( message... ){				\
	fprintf(stderr, ## message);			\
	exit(FAIL);					\
}

/* Define the macro to pass a test */
#define PASSTEST( ){					\
	fprintf(stderr, "Test %s passed\n", __FILE__);	\
	exit(PASS);					\
}

/* Define the standard check routines */
#define CHECK( _val, _assert ){				\
	__typeof__ (_val) __ret = (_assert);		\
	if (__ret != (_val)) {				\
		FAILTEST( "%s:%d: %s == %x != %x\n",	\
			__FILE__,			\
			__LINE__,			\
			#_assert,			\
			__ret,				\
			(_val));			\
	}						\
}

/* Minimum inits */
#define INIT_WAAAD() {					\
	CHECK( 0, conf_init()			);	\
	CHECK( 0, log_init(LOG_TO_CONSOLE)	);	\
	CHECK( 0, ext_init()			);	\
	CHECK( 0, dict_init()			);	\
	CHECK( 0, msg_init()			);	\
	CHECK( 0, meq_init()			);	\
	CHECK( 0, peer_init()			);	\
	CHECK( 0, sess_init()			);	\
	CHECK( 0, rt_init()			);	\
	CHECK( 0, disp_init()			);	\
	CHECK( 0, sec_init()			);	\
	parse_cmdline(argc, argv);			\
}

static inline void parse_cmdline(int argc, char * argv[]) {
	int c;
	while ((c = getopt (argc, argv, "dq")) != -1) {
		switch (c) {
			case 'd':	/* Increase verbosity of debug messages.  */
				g_conf->log_verbose++;
				break;
				
			case 'q':	/* Decrease verbosity then remove debug messages.  */
				g_conf->log_verbose--;
				break;
			
			default:	/* bug: option not considered.  */
				return;
		}
	}
}
 
#endif /* _TESTS_H */
